@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -33,6 +33,28 @@
 #define GCC_VERSION \
     (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
 
+//
+// CONFIG_ERROR_FUNCTION_ATTRIBUTE
+//
+
+// stev: CONFIG_ERROR_FUNCTION_ATTRIBUTE establishes whether GCC
+// supports the function attribute of the kind:
+//   `__attribute__ ((error(MESSAGE)))'.
+// This attribute is used for defining a powerful static (that
+// is compile-time) assertion checking macro.
+
+// The GCC's documentation of version 4.3.0 mentions for the
+// first time the presence of the 'error' function attribute:
+// http://gcc.gnu.org/onlinedocs/gcc-4.3.0/gcc/Function-Attributes.html.
+//
+// Note that I did not tested the attribute to work with GCC
+// of versions prior to the particular 4.3.4.
+#if GCC_VERSION < 40300
+#define CONFIG_ERROR_FUNCTION_ATTRIBUTE 0
+#else
+#define CONFIG_ERROR_FUNCTION_ATTRIBUTE 1
+#endif
+
 //
 // CONFIG_MEM_ALIGNOF
 //
@@ -128,5 +150,33 @@
 #error we need the NULL pointer to be represented as 0
 #endif
 
+//
+// CONFIG_DYNAMIC_LINKER
+//
+
+// stev: CONFIG_DYNAMIC_LINKER specifies the complete path to the
+// platform's dynamic linker; it is needed by the shared library
+// 'json.so' and by every compiled type library for to be able to 
+// to produce 'json.so' version numbers when issued by themselves;
+// the following shell function prints out dynamic linkers' full
+// path used by GCC on the target platform:
+//
+//   $ dynamic-linker() { local e=''; [[ "$1" == -+(32|64) ]] && { e="${1:1}"; shift; }; "${1:-gcc}" -Xlinker --verbose ${e:+-m$e} 2>/dev/null|sed -nr '/^found\s+ld.*\sat\s+([^ \t]+)\s*$/{s//\1/;p;q}'; }
+//
+//   $ dynamic-linker -64
+//   /lib64/ld-linux-x86-64.so.2
+//
+//   $ dynamic-linker -32
+//   /lib/ld-linux.so.2
+//
+// If CONFIG_DYNAMIC_LINKER is not defined, the library 'json.so'
+// and any compiled type library would build just fine, but with
+// no ability to produce 'json.so' version numbers by themselves.
+#ifdef __LP64__
+#define CONFIG_DYNAMIC_LINKER "/lib64/ld-linux-x86-64.so.2"
+#else
+#define CONFIG_DYNAMIC_LINKER "/lib/ld-linux.so.2"
+#endif
+
 #endif /* CONFIG_H */
 
