@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -15,11 +15,19 @@
 // You should have received a copy of the GNU General Public License
 // along with Json-Type.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 
+#if !defined(CONFIG_DYNAMIC_LINKER) && \
+    !defined(JSON_NO_LIB_MAIN)
+#define JSON_NO_LIB_MAIN
+#endif
+
 #ifndef JSON_NO_LIB_MAIN
+#include <unistd.h>
 #include "json.h"
 #endif
 
@@ -30,7 +38,7 @@ static const char program[] = STRINGIFY(PROGRAM);
 
 #ifndef JSON_NO_LIB_MAIN
 static const char license[] =
-"Copyright (C) 2016, 2017  Stefan Vargyas.\n"
+"Copyright (C) 2016, 2017, 2018  Stefan Vargyas.\n"
 "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n"
 "This is free software: you are free to change and redistribute it.\n"
 "There is NO WARRANTY, to the extent permitted by law.\n";
@@ -181,7 +189,14 @@ size_t json_timeval_to_usecs(
 
 #ifndef JSON_NO_LIB_MAIN
 
-int main(void)
+const char json_interp[]
+    __attribute__((section(".interp"))) =
+    CONFIG_DYNAMIC_LINKER;
+
+void json_main(void)
+    NORETURN;
+
+void json_main(void)
 {
     const char* b;
     struct json_lib_version_t v;
@@ -204,8 +219,9 @@ int main(void)
 #endif
         license
     );
+    fflush(stdout);
 
-    return 0;
+    _exit(0);
 }
 
 #endif // JSON_NO_LIB_MAIN
