@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -15,12 +15,21 @@
 // You should have received a copy of the GNU General Public License
 // along with Json-Type.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "config.h"
+
 #include <stdio.h>
 #include <ctype.h>
+#include <string.h>
+#include <stddef.h>
 
 #include "common.h"
 #include "pretty-print.h"
 #include "int-traits.h"
+#include "char-traits.h"
+#include "ptr-traits.h"
+#include "json-utf8.h"
+
+#define ISPRINT CHAR_IS_PRINT
 
 size_t pretty_print_len(char ch, size_t flags)
 {
@@ -46,7 +55,7 @@ size_t pretty_print_len(char ch, size_t flags)
             ? 2
             : 1;
     default:
-        return !isprint(ch)
+        return !ISPRINT(ch)
             ? 4
             : 1;
     }
@@ -90,7 +99,7 @@ const char* pretty_print_fmt(char ch, size_t flags)
             ? quoted_char
             : plain_char;
     default:
-        return !isprint(ch)
+        return !ISPRINT(ch)
             ? quoted_hex
             : plain_char;
     }
@@ -144,16 +153,12 @@ size_t pretty_print_string(
     return r;
 }
 
-void pretty_print_repr(
+static void pretty_print_repr_base(
     FILE* file, const uchar_t* str, size_t len,
-    enum pretty_print_repr_flags_t flags)
+    bool quote_non_print)
 {
-    if (flags & pretty_print_repr_print_quotes)
-        fputc('"', file);
-
     while (len --) {
         uchar_t c = *str ++;
-        size_t n;
 
         switch (c) {
         case '"':
@@ -178,32 +183,29 @@ void pretty_print_repr(
             fputs("\\t", file);
             break;
         default:
-            if (c < 0x20u)
-                n = 4;
-            else
-            if (!isascii(c)
-                && (flags & pretty_print_repr_quote_non_ascii))
-                n = 2;
-            else
-            if (iscntrl(c)
-                && (flags & pretty_print_repr_unicode_quote_cntrl))
-                n = 4;
-            else
-            if (!isprint(c)
-                && (flags & pretty_print_repr_quote_non_print))
-                n = 2;
-            else
-                n = 0;
-
-            if (n)
-                fprintf(file, "\\%c%0*x", n == 4 ? 'u' : 'x',
-                    SIZE_AS_INT(n), c);
+            if (quote_non_print && !ISPRINT(c))
+                fprintf(file, "\\x%02x", c);
             else
                 fputc(c, file);
         }
     }
+}
 
-    if (flags & pretty_print_repr_print_quotes)
+void pretty_print_repr(
+    FILE* file, const uchar_t* str, size_t len,
+    enum pretty_print_repr_flags_t flags)
+{
+    if (PRETTY_PRINT_REPR_FLAGS_IS(print_quotes))
+        fputc('"', file);
+
+    if (PRETTY_PRINT_REPR_FLAGS_IS(escape_utf8))
+        json_print_escaped_utf8(str, len,
+            PRETTY_PRINT_REPR_FLAGS_IS(surrogate_pairs), file);
+    else
+        pretty_print_repr_base(file, str, len,
+            PRETTY_PRINT_REPR_FLAGS_IS(quote_non_print));
+
+    if (PRETTY_PRINT_REPR_FLAGS_IS(print_quotes))
         fputc('"', file);
 }
 
