@@ -52,7 +52,7 @@
 
 static const char program[] = STRINGIFY(PROGRAM);
 static const char library[] = STRINGIFY(LIBRARY);
-static const char verdate[] = "0.8.8 -- 2017-12-02 10:12"; // $ date +'%F %R'
+static const char verdate[] = "0.8.9 -- 2018-04-09 13:09"; // $ date +'%F %R'
 
 static const char license[] =
 "Copyright (C) 2016, 2017, 2018  Stefan Vargyas.\n"
@@ -1353,6 +1353,56 @@ static size_t parse_debug_optarg(
 }
 #endif
 
+// stev: comment out the following define to disable
+// processing of long command line options using the
+// generated function 'lookup_long_opt' in header
+// 'long-opts-impl.h'
+#define CONFIG_GETOPT_LONG2
+
+#ifdef CONFIG_GETOPT_LONG2
+#include "long-opts-impl.h"
+
+static int getopt_long2(
+    int argc, char* const argv[],
+    const char* shorts, const struct option* longs,
+    int* index)
+{
+    const size_t n = sizeof(struct option);
+
+    static struct option l[2];
+    char *p, *q;
+    int k;
+
+    ASSERT(argc > 0);
+    ASSERT(argv != NULL);
+    ASSERT(shorts != NULL);
+    ASSERT(longs != NULL);
+
+    if (optind < argc) {
+        ASSERT(optind > 0);
+        p = argv[optind];
+
+        if (*p ++ == '-' && *p ++ == '-' && *p) {
+            if ((q = strchr(p, '=')))
+                *q = 0;
+
+            if ((k = lookup_long_opt(p)) >= 0) {
+                memcpy(l, &longs[k], n);
+                longs = l;
+            }
+            else
+                longs = l + 1;
+
+            if (q) *q = '=';
+        }
+    }
+
+    return getopt_long(argc, argv, shorts, longs, index);
+}
+
+#define getopt_long getopt_long2
+#endif // CONFIG_GETOPT_LONG2
+
 #if defined(DEBUG) && defined(DEBUG_GETOPT_LONG)
 
 static void print_getopt_long_debug(int opt)
@@ -1500,6 +1550,11 @@ const struct options_t* options(int argc, char* argv[])
         help_sizes_opt,
     };
 
+// stev: note that function 'lookup_long_opt' in header
+// 'long-opts-impl.h' needs to be regenerated each and
+// every time the variable 'long_opts' below changes,
+// in the case CONFIG_GETOPT_LONG2 has been defined!
+
     static struct option long_opts[] = {
         { "parse-only",             0,       0, quiet_parser_act },
         { "echo-parse",             0,       0, echo_parser_act },
