@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -119,6 +119,7 @@ static void obj_json_base_init_json(
         esc = CONF_BIT(obj_json_base_config_esc_type),
     };
     static const size_t types[] = {
+        CONF_DEF(allow_empty_input, obj),
         CONF_DEF(allow_literal_value, obj),
         CONF_DEF(allow_surrogate_pairs, obj|esc),
         CONF_DEF(disallow_non_ascii, obj),
@@ -128,6 +129,7 @@ static void obj_json_base_init_json(
         CONF_DEF(quote_text, esc),
     };
 
+    CONF_SET(allow_empty_input, empty_input);
     CONF_SET(allow_literal_value, liter_value);
     CONF_SET(allow_surrogate_pairs, surrogate_pairs);
     CONF_SET(disallow_non_ascii, ascii_only);
@@ -531,6 +533,42 @@ static bool text_address_get_offset(
     return true;
 }
 
+static bool file_read(
+    int fid, uchar_t* buf, size_t len,
+    size_t* result)
+{
+    size_t n = 0;
+
+    ASSERT(buf != NULL);
+    ASSERT(len > 0);
+
+    do {
+        ssize_t r;
+        size_t l;
+
+        r = read(fid, buf, len);
+        if (r < 0)
+            sys_error("reading failed");
+
+        if (r == 0) {
+            *result = n;
+            return true;
+        }
+
+        l = INT_AS_SIZE(r);
+        ASSERT(l <= len);
+
+        buf += l;
+        len -= l;
+
+        ASSERT_SIZE_ADD_NO_OVERFLOW(n, l);
+        n += l;
+    } while (len);
+
+    *result = n;
+    return false;
+}
+
 int obj_json_base_run(struct obj_json_base_t* this)
 {
     uchar_t b[this->buf_size];
@@ -650,21 +688,16 @@ int obj_json_base_run(struct obj_json_base_t* this)
     a.size ++;
 
     for (;;) {
-        ssize_t r;
+        size_t r;
+        bool f;
 
-        r = read(this->descr, b, this->buf_size);
-        if (r < 0)
-            sys_error("reading failed");
-        if (r == 0) {
-            if (t == after_error)
-                t = eof_error;
-            break;
-        }
+        f = file_read(
+                this->descr, b, this->buf_size, &r);
         if (this->echo_input)
-            fwrite(b, r, 1, stdout);
+            fwrite(b, 1, r, stdout);
 #ifdef DEBUG
         BUF_PRINT_DEBUG('$');
-        PRINT_DEBUG_BEGIN("$ r=%zd b[%p]=", r, b);
+        PRINT_DEBUG_BEGIN("$ f=%d r=%zu b[%p]=", f, r, b);
         pretty_print_string(
             stderr, b, r, pretty_print_string_quotes);
         PRINT_DEBUG_END();
@@ -694,8 +727,8 @@ int obj_json_base_run(struct obj_json_base_t* this)
         if (this->verbose &&
             !this->no_error) {
             enum mem_buf_op_t o;
-            size_t l = INT_AS_SIZE(r);
             uchar_t* p = b;
+            size_t l = r;
 
             BUF_PRINT_DEBUG('|');
             PRINT_DEBUG(
@@ -800,7 +833,7 @@ int obj_json_base_run(struct obj_json_base_t* this)
                     text_address_buf_update(
                         &a.buf, this->buf, p, l);
                 text_address_update(
-                    &a.input, b, INT_AS_SIZE(r));
+                        &a.input, b, r);
             }
             mem_buf_update(this->buf, p, l, o);
 
@@ -809,6 +842,12 @@ int obj_json_base_run(struct obj_json_base_t* this)
                 !a.size)
                 break;
         }
+
+        if (f) {
+            if (t == after_error)
+                t = eof_error;
+            break;
+        }
     }
 
     if (p)
