@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -93,7 +93,7 @@ static void print_repr_val(
     struct obj_json_printer_t* this, const uchar_t* str, size_t len,
     bool quotes)
 {
-    enum pretty_print_repr_flags_t f = 0;
+    enum pretty_print_repr_flags_t f = this->print_flags;
 
 #ifdef DEBUG
     print_debug(this);
@@ -101,8 +101,6 @@ static void print_repr_val(
 
     if (quotes)
         f |= pretty_print_repr_print_quotes;
-    if (this->quote_cntrl)
-        f |= pretty_print_repr_unicode_quote_cntrl;
     pretty_print_repr(OBJ_SUPER(this)->output, str, len, f);
     this->printed_out = true;
     this->skip_indent = false;
@@ -806,12 +804,14 @@ static void obj_json_printer_init(
         opts->print_type != options_terse_print_type;
     this->printed_out = false;
     this->skip_indent = false;
-    this->quote_cntrl = opts->quote_cntrl;
     this->print_dots = opts->print_dots;
     this->newline_sep = opts->newline_sep;
 #ifdef DEBUG
     this->debug = BITS_TRUNC_BITS(opts->debug_printer, debug_bits);
 #endif
+    this->print_flags =
+        (opts->surrogate_pairs ? pretty_print_repr_surrogate_pairs : 0) |
+        (opts->escape_utf8 ? pretty_print_repr_escape_utf8 : 0);
     this->indent_level = 0;
 }
 
